@@ -2,11 +2,12 @@
 
 import os
 import json
+import time
 import random
 import logging
 from copy import deepcopy
 
-from tornado import gen
+from tornado import gen, ioloop
 
 from litedfs.name.utils.append_log import AppendLogJson
 from litedfs.name.utils.listener import Connection
@@ -81,13 +82,16 @@ class FileSystemTree(object):
     _instance = None
     name = "file_system_tree"
 
-    def __new__(cls):
+    def __new__(cls, interval = 10):
         if not cls._instance:
             cls._instance = object.__new__(cls)
             cls._instance.tree = {F.children: {}, F.type: "root"}
             cls._instance.files = {}
             cls._instance.editlog = None
             cls._instance.status = "booting"
+            cls._instance.locks = {}
+            cls._instance.interval = interval
+            cls._instance.ioloop_service()
         return cls._instance
 
     @classmethod
@@ -97,6 +101,13 @@ class FileSystemTree(object):
         else:
             return None
 
+    def ioloop_service(self):
+        self.periodic_lock_service = ioloop.PeriodicCallback(
+            self.check_file_lock, 
+            self.interval * 1000
+        )
+        self.periodic_lock_service.start()
+
     @gen.coroutine
     def recover(self):
         self.status = "recovering"
@@ -107,6 +118,39 @@ class FileSystemTree(object):
         # TODO: synchronize between name node and data nodes
         self.status = "ready"
 
+    def set_file_lock(self, file_path, ttl = 60):
+        result = False
+        if file_path not in self.locks:
+            self.locks[file_path] = time.time() + ttl
+            result = True
+        return result
+
+    def unset_file_lock(self, file_path):
+        try:
+            if file_path in self.locks:
+                del self.locks[file_path]
+        except Exception as e:
+            LOG.exception(e)
+
+    def update_file_lock(self, file_path, ttl = 60):
+        result = False
+        if file_path in self.locks:
+            self.locks[file_path] = time.time() + ttl
+            result = True
+        return result
+
+    @gen.coroutine
+    def check_file_lock(self):
+        try:
+            now = time.time()
+            files = list(self.locks.keys())
+            for f in files:
+                if self.locks[f] < now:
+                    del self.locks[f]
+                    LOG.info("release write file lock: %s", f)
+        except Exception as e:
+            LOG.exception(e)
+
     def create(self, file_path, file_info):
         result = False
         dir_path, file_name = os.path.split(file_path)
@@ -315,8 +359,16 @@ class FileSystemTree(object):
             raise SourcePathNotExistsError("source path not exists: %s" % source_path)
         return result
 
-    def list_dir(self, directory_path, recursive = False):
-        result = []
+    def list_dir(self, directory_path, offset = 0, limit = 0, recursive = False, include_file = True, include_directory = True):
+        result = {"files": []}
+        if offset < 0:
+            offset = 0
+        if limit < 0:
+            limit = 0
+        result["offset"] = offset
+        result["limit"] = limit
+        files = []
+        dirs = []
         exists, file_type, file, _ = self.get_info(directory_path)
         if exists and file_type in (F.dir, "root"):
             if recursive:
@@ -329,7 +381,7 @@ class FileSystemTree(object):
                     child = {
                         "name": name,
                     }
-                    if file_type == F.file:
+                    if include_file and file_type == F.file:
                         file_id = file[F.children][name][F.id]
                         file_info = self.files[file_id]
                         child["type"] = "file"
@@ -343,11 +395,21 @@ class FileSystemTree(object):
                             child["current_replica"] = file_info["current_replica"]
                         if "replica" in file_info:
                             child["replica"] = file_info["replica"]
-                    elif file_type == F.dir:
+                        files.append(child)
+                    elif include_directory and file_type == F.dir:
                         child["type"] = "directory"
                         child["size"] = 0
                         child["id"] = ""
-                    result.append(child)
+                        dirs.append(child)
+                result["files"].extend(dirs)
+                result["files"].extend(files)
+                result["total"] = len(result["files"])
+                if offset > 0 and limit > 0:
+                    result["files"] = result["files"][offset:offset + limit]
+                elif offset > 0:
+                    result["files"] = result["files"][offset:]
+                elif limit > 0:
+                    result["files"] = result["files"][:limit]
             LOG.debug("list_dir: %s", result)
         return result
 
